"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/no-dynamic-delete */
/* eslint-disable dot-notation */
/* eslint-disable camelcase */
const url_1 = require("url");
const parseEndpoint_1 = __importDefault(require("./parseEndpoint"));
const normalizeQuery_1 = __importDefault(require("./normalizeQuery"));
const requests_1 = __importDefault(require("./requests"));
const catchHTTPErrors_1 = __importDefault(require("./catchHTTPErrors"));
const DataController_1 = __importDefault(require("../DataController"));
const createEndpoint = (type, endpoint) => {
    const data = DataController_1.default.getInstance();
    return async (options) => {
        const { body, pathParameters, query, } = options;
        let parsedEndpoint = null;
        let response = null;
        if (pathParameters)
            parsedEndpoint = parseEndpoint_1.default(endpoint, pathParameters);
        if (query) {
            if (query['api_key'])
                delete query['api_key'];
            if (query['language'])
                delete query['language'];
        }
        const readyQuery = new url_1.URLSearchParams({
            api_key: data.get('apiKey'),
            language: data.get('language'),
        });
        if (query) {
            normalizeQuery_1.default(query).forEach(entry => {
                readyQuery.append(entry[0], entry[1]);
            });
        }
        try {
            switch (type) {
                case 'GET': {
                    response = await requests_1.default(parsedEndpoint, { searchParams: readyQuery });
                    break;
                }
                case 'POST': {
                    response = await requests_1.default.post(endpoint, {
                        searchParams: readyQuery,
                        json: body,
                    });
                    break;
                }
                case 'DELETE': {
                    response = await requests_1.default.delete(endpoint, {
                        searchParams: readyQuery,
                        json: body,
                    });
                    break;
                }
            }
            return {
                data: JSON.parse(response.body),
                headers: response.headers,
            };
        }
        catch (error) {
            catchHTTPErrors_1.default(error);
            throw error;
        }
    };
};
exports.default = (endpoints) => {
    const root = {};
    endpoints.forEach(endpoint => {
        root[endpoint.name] = createEndpoint(endpoint.type, endpoint.endpoint);
    });
    return root;
};
